package com.lantone.filter;

import com.alibaba.fastjson.JSON;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.diagbot.util.JwtUtil;
import com.lantone.config.ExclusionUrl;
import com.lantone.facade.TokenFacade;
import com.lantone.security.dto.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: AuthGlobalFilter
 * @Description: 将登录用户的JWT转化成用户信息的全局过滤器
 * @Author songxl
 * @Date 2021/6/29
 * @Version 1.0
 */
@Component
@Slf4j
public class AuthGlobalFilter implements GlobalFilter, Ordered {
    private static final String URI = "/v2/api-docs";
    @Autowired
    private ExclusionUrl exclusionUrl;
    @Autowired
    TokenFacade tokenFacade;
    /**
     * @Author songxl
     * @Description 全局过滤器过滤方法
     * @Date  2021/6/29
     * @Param [exchange, chain]
     * @Return reactor.core.publisher.Mono<java.lang.Void>
     * @MethodName filter
     */
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {

        ServerHttpRequest request = exchange.getRequest();
        ServerHttpResponse response = exchange.getResponse();
        //1.swagger请求不要拦截
        String path = request.getURI().getPath();
        log.info("request path:{}", path);
        if (StringUtils.endsWithIgnoreCase(path, URI)) {
            return chain.filter(exchange);
        }
        //2、白名单无需权限判断
        if (isExclusionUrl(path)) {
            return chain.filter(exchange);
        }
        String headerToken = request.getHeaders().getFirst("Authorization");
        //3、只要带上了token， 就需要判断Token是否有效
        if (!StringUtils.isEmpty(headerToken)) {
            headerToken = headerToken.replaceFirst("Bearer ", "");
            if (!tokenFacade.verifyToken(headerToken, 1)) {
                return getVoidMono(response, 401, "token无效");
            }
            //4、判断请求的URL是否有权限
            boolean permission = hasPermission(headerToken, path);
            if (!permission) {
                return getVoidMono(response, 403, "无访问权限");
            }
            return chain.filter(exchange);
        }
        return getVoidMono(response, 401, "请求token为空");
    }

    @Override
    public int getOrder() {
        return 0;
    }
    /**
     * @Author songxl
     * @Description 白名单请求判断
     * @Date  2021/7/1
     * @Param [path]
     * @Return boolean
     * @MethodName isExclusionUrl
     */
    private boolean isExclusionUrl(String path) {
        List<String> exclusions = exclusionUrl.getUrl();
        if (exclusions.size() == 0) {
            return false;
        }
        for (String action:exclusions){
            if (StringUtils.endsWithIgnoreCase(path, action)) {
                return true;
            }
        }
        return false;
    }
    /**
     * @Author songxl
     * @Description 从token获取权限信息，判断token是否有该请求权限
     * @Date  2021/7/1
     * @Param [headerToken, path]
     * @Return boolean
     * @MethodName hasPermission
     */
    private boolean hasPermission(String headerToken, String path) {
        String url, method;
        try {
            DecodedJWT jwt = JwtUtil.decodedJWT(headerToken);
            Map<String, Claim> claims = jwt.getClaims();
            Claim claim = (Claim) claims.get("authorities");
            String[] permissinos = claim.asArray(String.class);
            for (Object permission : permissinos) {
                String[] authority = (permission + "").split(";");
                url = authority[0];
                method = authority[1];
                if (StringUtils.endsWithIgnoreCase(path, url)) {
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
    /**
     * @Author songxl
     * @Description 请求返回拼装方法
     * @Date  2021/7/1
     * @Param [response, i, msg]
     * @Return reactor.core.publisher.Mono<java.lang.Void>
     * @MethodName getVoidMono
     */
    private Mono<Void> getVoidMono(ServerHttpResponse response, int i, String msg) {
        response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        response.setStatusCode(HttpStatus.OK);
        Result failed = Result.failure(i, msg);
        byte[] bits = JSON.toJSONString(failed).getBytes();
        DataBuffer buffer = response.bufferFactory().wrap(bits);
        return response.writeWith(Mono.just(buffer));
    }
}
